package ml;

import weka.core.DataContainer;
import ml.active.LearnersFactory;
import ml.active.KFF;
import ml.active.XActiveLearner;
import ml.util.config.XmlReadOnlyConfig;

/**
 * <p>Title: Active Learning</p>
 *
 * <p>Description: </p>
 *
 * <p>Copyright: Copyright (c) 2005</p>
 *
 * <p>Company: </p>
 *
 * @author not ron
 * @version 1.0
 */
public class Learner {
    private static final String CONFIG_DEFAULT_FILE = "activeLearning/config/config.xml";
    private static final String H_DELIM = "----------------------------";

    private XActiveLearner learner;

    public Learner() {
    }

    /**
     *
     * @param learnerType String one of: SIMPLE, SELF_CONF, KFF, BALANCED_EE
     */
    public void init(String learnerType) {
        try {
            XmlReadOnlyConfig config = new XmlReadOnlyConfig();
            config.init(CONFIG_DEFAULT_FILE);
            LearnersFactory factory = new LearnersFactory();
            learner = factory.create(learnerType,config);
        }
        catch(Exception e){
            handleException(e);
        }
    }

    public void learn(DataContainer data) {
        try {
            learner.buildClassifier(data);
        } catch (Exception e) {
            handleException(e);
        }
    }

    public double[] classify(DataContainer data) throws Exception {
        double[] res = new double[data.size()];
        for (int i = 0, test = data.size(); i < test; ++i) {
            res[i] = learner.classifyInstance(data.getInstance(i));
        }
        return res;
    }

    public int indexToQuery(DataContainer pool) throws Exception {
        return learner.instanceToQuery(pool);
    }

    public DataContainer create(String dataFile) throws Exception {
        return learner.create(dataFile);
    }

    public DataContainer create(double[][] attributes, double[] labels) throws
            Exception {
        return learner.create(attributes, labels);
    }

    private void handleException(Exception e) {
        System.out.println(H_DELIM);
        System.out.println("Learner failed with:");
        e.printStackTrace();
        System.out.println(H_DELIM);
    }

}
