package ml.active;

import weka.active.*;
import weka.core.*;

import weka.classifiers.libsvm.*;
import ml.util.config.ReadOnlyConfiguration;

/**
 * <p>Title: Active Learning</p>
 *
 * <p>Description: </p>
 *
 * <p>Copyright: Copyright (c) 2005</p>
 *
 * <p>Company: </p>
 *
 * @author not attributable
 * @version 1.0
 */
abstract public class XSvmActiveLearner extends XActiveLearner {

    private SvmLib svm;

    public XSvmActiveLearner() {
        svm = new SvmLib();
    }

    /**
     * buildClassifier
     *
     * @param data DataContainer
     * @throws Exception
     * @todo Implement this weka.active.ActiveLearner method
     */
    public void buildClassifier(DataContainer data) throws Exception {
        svm.buildClassifier(data);
    }

    /**
     * classifyInstance
     *
     * @param instance InstanceContainer
     * @return double
     * @throws Exception
     * @todo Implement this weka.active.ActiveLearner method
     */
    public double classifyInstance(InstanceContainer instance) throws Exception {
        return svm.classifyInstance(instance);
    }

    /**
     * setDataAndPool
     *
     * @param data Instances
     * @param pool DataContainer
     * @throws Exception
     * @todo Implement this weka.active.ActiveLearner method
     */
    public void setDataAndPool(Instances data, DataContainer pool) throws
            Exception {
        throw new RuntimeException("method not supported");
    }

    /**
     * startActiveLearningSession
     *
     * @throws Exception
     * @todo Implement this weka.active.ActiveLearner method
     */
    public void startActiveLearningSession() throws Exception {
        throw new RuntimeException("method not supported");
    }

    public void init(DataContainer initialData) throws Exception {
        buildClassifier(initialData);
    }

    /**
     * creates a DataContainer corresponding instances
     * @param instances Instances
     * @return DataContainer
     * @throws Exception
     */
    public DataContainer create(Instances instances) throws Exception {
        return svm.convertData(instances);
    }

    /**
     * creates DataContainer corresponding `data'. the scaling of the values
     * corresponds the min-max values of `former'
     *
     * @param data Instances for DataContainer
     * @param former DataContainer determines the scaling min-max values
     * @return DataContainer
     * @throws Exception
     */
    public DataContainer create(Instances data, DataContainer former) throws
            Exception {
        return svm.convertData(data, former);
    }


    /**
     * Creates a DataContainer corresponding the data in file `fileName'.
     * @param fileName String
     * @return DataContainer
     */
    public DataContainer create(String fileName) throws Exception {
        Instances instances = new Instances(fileName);
        return svm.convertData(instances);
    }

    /**
     * Creates a DataContainer corresponding the data in file `fileName'.
     * @param attributes double[][]
     * @param labels double[]
     * @return DataContainer
     * @throws Exception
     */
    public DataContainer create(double[][] attributes, double[] labels) throws
            Exception {
        return svm.convertData(attributes, labels);
    }

    public void init(ReadOnlyConfiguration config) throws CreationException {

        String sVal;
        try {
            sVal = config.get("svm/gamma");
            double gamma = Double.parseDouble(sVal);
            this.svm.setGamma(gamma);
        } catch (Exception e) {
            throw new CreationException("Failed to parse gamma");
        }

        try {
            sVal = config.get("svm/c");
            double c = Double.parseDouble(sVal);
            this.svm.setCost(c);
        } catch (Exception e) {
            throw new CreationException("Failed to parse c");
        }

    }


    protected SvmLib getSvm() {
        return this.svm;
    }


}
