package weka.active;

import java.util.*;
import java.io.*;
import weka.active.*;
import weka.classifiers.libsvm.*;
import weka.core.*;

public class SvmErrorReductionInfo extends weka.active.SvmActiveLearner
                                   implements RandomUser, OptionHandler
{            
    private int m_minSampleSize = 10;
    private int m_maxSampleSize = 100;
    private int m_currSampleSize;
    private int m_dec = 1;
    
    public void startActiveLearningSession() throws Exception
    {                  
        super.startActiveLearningSession();                            
        this.m_currSampleSize = m_maxSampleSize;
    } 
    
    public int instanceToQuery(DataContainer pool) throws Exception
    {
        int res = super.instanceToQuery(pool);             
        if (res != -1) {
            return res;
        }
        
        int i, sampleSz, poolSz = pool.size();        
        DataContainer sampledPool;
        int[] indexes;
        
        if (m_currSampleSize <= m_minSampleSize) {
            sampleSz = m_minSampleSize; 
            m_currSampleSize = m_minSampleSize;
        } else {
            sampleSz = m_currSampleSize;
            m_currSampleSize -= m_dec;
        } 
        if (poolSz < sampleSz) {
            sampleSz = poolSz; 
            sampledPool = pool;
            indexes = new int[sampleSz];
            for (i=0; i<sampleSz; i++) {
                indexes[i] = i;            
            }
        } else {            
            indexes = new int[sampleSz];
            sampledPool = this.sampleFromPool(pool, 
                                              sampleSz, 
                                              this.m_rndm,
                                              indexes);            
        }               
        System.out.println("Sample size=" + sampleSz);
                    
        res = this.instanceToQueryErrorReductionMode(sampledPool);
        if (res != -1) {  
            System.out.println("\tquerying in error reduction mode. indx=" + sampledPool.getInstance(res).getIndex());  
            return indexes[res];  
        }        
        throw new Exception("no res");
    }
           
    private int instanceToQueryErrorReductionMode(DataContainer pool) throws Exception
    {        
        int i, best=-1, currCls, poolSz = pool.size();
        InstanceContainer ins;
        double currConf, minLoss = Double.MAX_VALUE, currLoss;
        
        // find the instance with lowest confidence
        for (i=0; i<poolSz; i++) {
            currLoss = 0.0;
            ins = pool.getInstance(i);
            currConf = Math.abs(this.m_svm.confindenceOnInstance(ins));            
            currConf = this.logisticRegression(currConf);            
            currCls = (int)(this.m_svm.classifyInstance(ins));
            currLoss += (this.expectedLogLoss(ins, currCls, pool) * currConf);           
            currCls = ((currCls == 0) ? 1 : 0);
            currConf = 1.0 - currConf;
            currLoss += (this.expectedLogLoss(ins, currCls, pool) * currConf);            
            if (Utils.sm(currLoss, minLoss)) {
                minLoss = currLoss;
                best = i;                
            }                           
        }        
        return best;
    }
    
    public double[] distributionToQuery(DataContainer pool) throws Exception 
    {
        int i, newIndx, currCls;
        int sampleSz, poolSz = pool.size();
        InstanceContainer ins;
        DataContainer sampledPool;
        double currConf, currLoss;
        double sum, lowestLoss = Double.MAX_VALUE, highestLoss=-Double.MAX_VALUE;
        double[] prob = new double[poolSz];        
        int[] indexes;
        
        System.out.println("error-reduction dist");
        for (i=0; i<poolSz; i++) {
            prob[i] = 0.0;            
        }
        // sample                         
        if (m_currSampleSize <= m_minSampleSize) {
            sampleSz = m_minSampleSize; 
            m_currSampleSize = m_minSampleSize;
        } else {
            sampleSz = m_currSampleSize;
            m_currSampleSize -= m_dec;
        } 
        if (poolSz < sampleSz) {
            sampleSz = poolSz; 
            sampledPool = pool;
            indexes = new int[sampleSz];
            for (i=0; i<sampleSz; i++) {
                indexes[i] = i;            
            }
        } else {            
            indexes = new int[sampleSz];
            sampledPool = this.sampleFromPool(pool, 
                                              sampleSz, 
                                              this.m_rndm,
                                              indexes);            
        }
        System.out.println("Sample size=" + sampleSz);
        
        // find the instance with lowest confidence                
        for (i=0; i<sampleSz; i++) {
            currLoss = 0.0;
            ins = sampledPool.getInstance(i);
            currConf = Math.abs(this.m_svm.confindenceOnInstance(ins));            
            currConf = this.logisticRegression(currConf);            
            currCls = (int)(this.m_svm.classifyInstance(ins));
            currLoss += (this.expectedLogLoss(ins, currCls, sampledPool) * currConf);            
            currCls = ((currCls == 0) ? 1 : 0);
            currConf = 1.0 - currConf;
            currLoss += (this.expectedLogLoss(ins, currCls, sampledPool) * currConf);   
            
            newIndx = indexes[i];
            prob[newIndx] = currLoss;            
            if (Utils.sm(prob[newIndx], lowestLoss)) {
                lowestLoss = prob[newIndx];
            }
            if (Utils.gr(prob[newIndx], highestLoss)) {
                highestLoss = prob[newIndx];
            }                  
        }               
        for (i=0; i<sampleSz; i++) {
            newIndx = indexes[i];
            if (Utils.eq(lowestLoss, highestLoss)) {                
                prob[newIndx] = 0.5;
            } else {                            
                prob[newIndx] = (prob[newIndx] - lowestLoss) / (highestLoss - lowestLoss);
                if (Utils.sm(prob[newIndx], 0.0)) {
                    prob[newIndx] = 0.0;
                } 
                if (Utils.gr(prob[newIndx], 1.0)) {
                    prob[newIndx] = 1.0;
                }
            }            
        }              
        // scale up             
        double scale = 100;        
        for (i=0; i<sampleSz; i++) {
            newIndx = indexes[i];
            prob[newIndx] *= scale;            
        }              
        // exponential scale 
        sum = 0.0;        
        for (i=0; i<sampleSz; i++) {
            newIndx = indexes[i];
            prob[newIndx] = Math.exp(-1.0 * prob[newIndx]);
            sum += prob[newIndx];                        
        }              
        for (i=0; i<sampleSz; i++) {
            newIndx = indexes[i];
            prob[newIndx] = prob[newIndx] / sum;              
        }        
        for (i=0; i<sampleSz; i++) {
            newIndx = indexes[i];
            if (Utils.grOrEq(prob[newIndx], 0.01)) {                
                System.out.println("prob[" + newIndx + "]=" + prob[newIndx]);
            }            
        }        
        // check
        sum = 0.0;
        for (i=0; i<poolSz; i++) {             
            sum += prob[i];
        } 
        if (!Utils.eq(sum, 1.0)) throw new Exception("sum probs is not 1.0 - sum=" + sum);
        return prob;
    }

    private double expectedLogLoss(InstanceContainer insToAdd, int cls, DataContainer pool) throws Exception
    {
        int i, poolSz = pool.size();
        double currConf, sum;
        InstanceContainer ins;
        
        insToAdd.setClassValue(cls);
        this.m_fullTrainingSet.addInstance(insToAdd);
        this.m_svm.buildClassifier(this.m_fullTrainingSet);
        sum = 0.0;
        for (i=0; i<poolSz; i++) {
            ins = pool.getInstance(i);
            currConf = Math.abs(this.m_svm.confindenceOnInstance(ins));
            currConf = this.logisticRegression(currConf);
            if (!Utils.eq(currConf, 0)) {                
                sum += (currConf * Math.log(currConf));
            }
            currConf = 1.0 - currConf;
            if (!Utils.eq(currConf, 0)) {                
                sum += (currConf * Math.log(currConf));
            }
            if (Utils.gr(sum, 0.0)) throw new Exception("sum is positive");
        }
        sum /= ((double)poolSz);
        sum = Math.abs(sum);        
        this.m_fullTrainingSet.deleteInstance(this.m_fullTrainingSet.size()-1);
        this.m_svm.buildClassifier(this.m_fullTrainingSet);
        return sum;
    }
    
    private double logisticRegression(double conf) throws Exception
    {
        if (Utils.sm(conf, 0.0)) throw new Exception("conf is negative" + conf);
        double res = (1.0 / (1.0 + Math.exp(-1.0 * conf)));
        if (Utils.sm(res, 0.5) || Utils.gr(res, 1.0)) throw new Exception("logic reg is not in [0.5, 1]");
        return res;
    }
            
    private String[] copyOptions(String[] options)
    {
        int i;                
        String[] optionsCopy = new String[options.length];
        for (i=0; i<optionsCopy.length; i++) {
            optionsCopy[i] = new String(options[i]);
        }   
        return optionsCopy;
    }
    
    public void setOptions(String options[]) throws Exception
    {
        int i;
        super.setOptions(options);       
        String SString = Utils.getOption('S', options);
        if (SString.length() != 0) {
            this.setSeed((new Integer(SString)).intValue());            
        }
        String[] svmOptions = Utils.partitionOptions(options);        
        this.m_svm.setOptions(svmOptions);
        Utils.checkForRemainingOptions(options);
    }

    public Enumeration listOptions()
    {
        FastVector newVector = new FastVector(1);        	         
        newVector.addElement(new Option(
         "\tRandom seed.\n"
         +"\t(Default is 1234567)",
         "S", 1, "-S <seed>"));
	return newVector.elements();
    }

    public String[] getOptions()
    {
       String [] options = new String [2];
       int current = 0;
       options[current++] = "-S"; options[current++] = "" + this.m_seed;       
       return options;
    }  
    
    public String toString() 
    {
        StringBuffer msg = new StringBuffer();
        msg.append("SvmSimpleCluster Active Learner\n-----------------------");
        msg.append("The SVM used:\n" + this.m_svm.toString());
        return msg.toString();
    } 
    
    public static void main(String[] args) {
        try {
            System.out.println(ActiveEvaluationPlotter.evaluateModelAndPlot(new SvmErrorReductionInfo(), args));
            ActiveEvaluationPlotter.Plot(m_selfTestInfo, "entropy");                                           
        } catch (Exception e) {            
            System.err.println(e.getMessage());            
            e.printStackTrace();
        }  
    }        
}

