package weka.active;

import java.util.*;
import java.io.*;
import weka.active.*;
import weka.classifiers.libsvm.*;
import weka.core.*;

public class SvmSimpleInfo extends weka.active.SvmActiveLearner
                           implements RandomUser, OptionHandler
{                        
    public int instanceToQuery(DataContainer pool) throws Exception
    {
        int res = super.instanceToQuery(pool);             
        if (res != -1) {
            return res;
        }        
        res = this.instanceToQuerySimpleMode(pool);                
        if (res != -1) {  
            System.out.println("\tquerying in simple mode. indx=" + pool.getInstance(res).getIndex());              
            return res;                               
        }        
        throw new Exception("no res");
    }
           
    private int instanceToQuerySimpleMode(DataContainer pool) throws Exception
    {        
        int i, closest=-1;
        int size = pool.size();
        InstanceContainer ins;
        double minConf = Double.MAX_VALUE, currConf;  
        
        // find the instance with lowest confidence
        for (i=0; i<size; i++) {
            ins = pool.getInstance(i);
            currConf = Math.abs(this.m_svm.confindenceOnInstance(ins));
            if (Utils.smOrEq(currConf, minConf)) {
                minConf = currConf;
                closest = i;
            }            
        }
        return closest;
    }
    
    public double[] distributionToQuery(DataContainer pool) throws Exception
    {
        int i;
        int size = pool.size();
        InstanceContainer ins;        
        double currConf, sum;  
        double lowestConf = Double.MAX_VALUE, highestConf=-Double.MAX_VALUE;
        double[] prob = new double[size];
                
        System.out.println("simple dist");
        // find all instances confidence and scale by min-max      
        for (i=0; i<size; i++) {
            ins = pool.getInstance(i);
            currConf = Math.abs(this.m_svm.confindenceOnInstance(ins));
            prob[i] = currConf;              
            if (Utils.sm(prob[i], lowestConf)) {
                lowestConf = prob[i];
            }
            if (Utils.gr(prob[i], highestConf)) {
                highestConf = prob[i];
            }                  
        }                
        for (i=0; i<size; i++) {
            if (Utils.eq(lowestConf, highestConf)) {
                prob[i] = 0.5;
            } else {                            
                prob[i] = (prob[i] - lowestConf) / (highestConf - lowestConf);
                if (Utils.sm(prob[i], 0.0)) {
                    prob[i] = 0.0;
                } 
                if (Utils.gr(prob[i], 1.0)) {
                    prob[i] = 1.0;
                }
            }            
        }              
        // scale up             
        double scale = 100;        
        for (i=0; i<size; i++) {
            prob[i] *= scale;            
        }              
        // exponential scale 
        sum = 0.0;        
        for (i=0; i<size; i++) {
            prob[i] = Math.exp(-1.0 * prob[i]);                                  
            sum += prob[i];
        }                
        for (i=0; i<size; i++) {
            prob[i] = prob[i] / sum;              
        }
        for (i=0; i<size; i++) {
            if (Utils.grOrEq(prob[i], 0.01)) {                
                System.out.println("prob[" + i + "]=" + prob[i]);
            }            
        }        
        // check
        sum = 0.0;
        for (i=0; i<size; i++) {            
            sum += prob[i];
        } 
        if (!Utils.eq(sum, 1.0)) throw new Exception("sum probs is not 1.0 - sum=" + sum);
        return prob;
    }        
            
    private String[] copyOptions(String[] options)
    {
        int i;                
        String[] optionsCopy = new String[options.length];
        for (i=0; i<optionsCopy.length; i++) {
            optionsCopy[i] = new String(options[i]);
        }   
        return optionsCopy;
    }
    
    public void setOptions(String options[]) throws Exception
    {
        int i;
        super.setOptions(options);       
        String SString = Utils.getOption('S', options);
        if (SString.length() != 0) {
            this.setSeed((new Integer(SString)).intValue());            
        }
        String[] svmOptions = Utils.partitionOptions(options);        
        this.m_svm.setOptions(svmOptions);
        Utils.checkForRemainingOptions(options);
    }

    public Enumeration listOptions()
    {
        FastVector newVector = new FastVector(1);        	         
        newVector.addElement(new Option(
         "\tRandom seed.\n"
         +"\t(Default is 1234567)",
         "S", 1, "-S <seed>"));
	return newVector.elements();
    }

    public String[] getOptions()
    {
       String [] options = new String [2];
       int current = 0;
       options[current++] = "-S"; options[current++] = "" + this.m_seed;       
       return options;
    }  
    
    public String toString() 
    {
        StringBuffer msg = new StringBuffer();
        msg.append("SvmSimpleCluster Active Learner\n-----------------------");
        msg.append("The SVM used:\n" + this.m_svm.toString());
        return msg.toString();
    } 
    
    public static void main(String[] args) {
        try {
            System.out.println(ActiveEvaluationPlotter.evaluateModelAndPlot(new SvmSimpleInfo(), args));                    
            ActiveEvaluationPlotter.Plot(m_selfTestInfo, "entropy");                        
        } catch (Exception e) {            
            System.err.println(e.getMessage());            
            e.printStackTrace();
        }  
    }        
}
